//
//  WebSiteScreenCapture.m
//  WebSiteScreenCapture
//
//  Created by Watanabe Toshinori on 2012/07/18.
//  Copyright (c) 2012 FLCL.jp All rights reserved.
//

#import <QuartzCore/QuartzCore.h>
#import "WebSiteScreenCapture.h"

@interface WebSiteScreenCapture ()
@property (nonatomic, strong) UIWebView *webView;
@property (nonatomic, copy) WebScreenCaptureHandler completetion;
@end

@interface WebSiteScreenCapture (UIWebViewDelegate) <UIWebViewDelegate>
@end


@implementation WebSiteScreenCapture

@synthesize webView = _webView;
@synthesize completetion = _completetion;


#pragma mark - Memory management

+ (id)sharedWebSiteScreenCapture
{
    static dispatch_once_t onceQueue;
    static WebSiteScreenCapture *webSiteScreenCapture = nil;
    
    dispatch_once(&onceQueue, ^{ webSiteScreenCapture = [[self alloc] init]; });
    return webSiteScreenCapture;
}


#pragma mark - Private method

- (void)captureWithURL:(NSString *)url handler:(WebScreenCaptureHandler)handler
{
    [self captureWithURL:url size:[UIScreen mainScreen].bounds.size handler:handler];
}

- (void)captureWithURL:(NSString *)url size:(CGSize)size handler:(WebScreenCaptureHandler)handler
{
    self.completetion = handler;

    self.webView = [[UIWebView alloc] initWithFrame:CGRectMake(0, 0, size.width, size.height)];
    self.webView.delegate = self;
    self.webView.scalesPageToFit = YES;

    [self.webView loadRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:url]]];
}


#pragma mark - Public method

+ (void)captureWithURL:(NSString *)url handler:(WebScreenCaptureHandler)handler
{
    [[WebSiteScreenCapture sharedWebSiteScreenCapture] captureWithURL:url handler:handler];
}

+ (void)captureWithURL:(NSString *)url size:(CGSize)size handler:(WebScreenCaptureHandler)handler;
{
    [[WebSiteScreenCapture sharedWebSiteScreenCapture] captureWithURL:url size:size handler:handler];
}

@end


#pragma mark -
@implementation WebSiteScreenCapture (UIWebViewDelegate)

- (void)webViewDidFinishLoad:(UIWebView *)webView
{
    UIGraphicsBeginImageContextWithOptions(self.webView.frame.size, NO, 0.0);
    CGContextRef context = UIGraphicsGetCurrentContext();
    [self.webView.layer renderInContext:context];
    UIImage *image = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    
    self.webView.delegate = nil;
    
    self.completetion(image, nil);
}

- (void)webView:(UIWebView *)webView didFailLoadWithError:(NSError *)error
{
    self.webView.delegate = nil;
    
    self.completetion(nil, error);
}

@end
